#
# Copyright (c) 2000 Going Virtual, LLC. All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# Contact Information:
#
# Going Virtual, LLC
# 10800 Independence Pointe Pkwy
# Matthews, NC 28105
# www.goingv.com
#
# Robert E. Tierney 
# bob.tierney@goingv.com 
# 1 (704) 849-0731 x109
#
#
__doc__ = """Going Virtual Interbase Database Interface

Python Database API Specification 2.0 Compliant

This module defines the exception classes used by this Interbase Adapter.

$Id: gvibExceptions.py,v 1.4 2000/07/11 20:04:00 btierney Exp $
"""
__version__ = '$Revision: 1.4 $'[11:-2]

################################################################################
# 
# The exception hierarchy, as shown by indentation, is:
# 
# Standard Error
#   Warning
#   Error
#     InterfaceError
#     DatabaseError
#     DataError
#     OperationalError
#     IntegrityError
#     InternalError
#     ProgrammingError
#     NotSupportedError
# 
# There are 4 data elements associated with an exception: 1) SQL Code, 2) SQL
# Message, 3) Interbase Code, and 4) Interbase Messages.  The SQL Code and SQL
# Message are self evident.  The Interbase Code is the long value of the 
# Interbase error that is most likely to be the problem.  The Interbase Messages
# are all of the messages that are placed into the status vector by the API
# library function that fails.
# 
# The 'value' of an exception is the Interbase messages concatenated together
# and separated by '--'.  In most cases these messages will be the most 
# informative. If an exception is raised for any reason other than a failing
# API library function call, the SQL and Interbase codes will be 1 for Warning 
# and -1 for Error and its subclasses. In addition, the SQL message will be an
# empty string.  The Interbase message attribute will contain the reason for
# raising the exception.
# 
# When an API function fails a lookup is made in a dictionary to determine what
# type of exception to throw.  The keys in the dictionary are SQL codes and 
# the values are exception classes.  This dictionary may need tweaked.
# 
# Example:
# 
# import gvib
# import gvibExceptions
# 
# try:
#   cursor.execute(...)
# except gvibExceptions.Error, e:  # catch all errors
#   sqlcode = e.getSqlCode()
#   if sqlcode == -803:
#     print "The dreaded -803 strikes again!'
# 
#   print sqlcode
#   print e.getSqlMsg()
#   print e.getIBMsg()             # concatenated messages
#   print e.getIBMsgList()         # tuple of messages
# 
################################################################################

import gvibBase

from exceptions import StandardError
from string import split

separator = " -- "

class Warning(StandardError):
    """
    Warning Class -
    """

    def __init__(self, sqlcode = 1, sqlmsg = '', ibcode = 1, ibmsg = ''):
        self.__sqlcode = sqlcode
	self.__sqlmsg  = sqlmsg
        self.__ibcode  = ibcode
	self.__ibmsg   = ibmsg

    def __str__(self):
        return self.__ibmsg

    def getSqlCode(self):
        return self.__sqlcode

    def getSqlMsg(self):
        return self.__sqlmsg

    def getIBCode(self):
        return self.__ibcode

    def getIBMsg(self):
        return self.__ibmsg

    def getIBMsgList(self):
        return split(self.__ibmsg, separator)



class Error(StandardError):
    """
    Error -
    """

    def __init__(self, sqlcode = -1, sqlmsg = '', ibcode = -1, ibmsg = ''):
        self.__sqlcode = sqlcode
	self.__sqlmsg  = sqlmsg
        self.__ibcode  = ibcode
	self.__ibmsg   = ibmsg

    def __str__(self):
        return self.__ibmsg

    def getSqlCode(self):
        return self.__sqlcode

    def getSqlMsg(self):
        return self.__sqlmsg

    def getIBCode(self):
        return self.__ibcode

    def getIBMsg(self):
        return self.__ibmsg

    def getIBMsgList(self):
        return split(self.__ibmsg, separator)



class InterfaceError(Error):
    """
    InterfaceError Class - Subclass of Error class.
    """
    pass


class DatabaseError(Error):
    """
    DatabaseError Class - Subclass of Error class.
    """
    pass


class InternalError(DatabaseError):
    """
    InternalError Class - Subclass of Error class.
    """
    pass


class OperationalError(DatabaseError):
    """
    OperationalError Class - Subclass of Error class.
    """
    pass


class ProgrammingError(DatabaseError):
    """
    ProgrammingError Class - Subclass of Error class.
    """
    pass


class IntegrityError(DatabaseError):
    """
    IntegrityError Class - Subclass of Error class.
    """
    pass


class DataError(DatabaseError):
    """
    DataError Class - Subclass of Error class.
    """
    pass


class NotSupportedError(DatabaseError):
    """
    NotSupportedError Class - Subclass of Error class.
    """
    pass


#
# Table to map SQL error codes to exception types.
#
_ExceptionTable = {
     0 : None,
   100 : Warning,
   101 : Warning,
   -84 : InterfaceError,
  -103 : DataError,
  -104 : InterfaceError,
  -150 : ProgrammingError,
  -151 : ProgrammingError,
  -155 : ProgrammingError,
  -157 : ProgrammingError,
  -158 : ProgrammingError,
  -162 : NotSupportedError,
  -170 : ProgrammingError,
  -171 : ProgrammingError,
  -172 : NotSupportedError,
  -204 : Error,
  -205 : ProgrammingError,
  -206 : ProgrammingError,
  -208 : ProgrammingError,
  -219 : ProgrammingError,
  -239 : InternalError,
  -260 : InternalError,
  -281 : ProgrammingError,
  -282 : InternalError,
  -283 : InternalError,
  -284 : InternalError,
  -291 : IntegrityError,
  -292 : IntegrityError,
  -293 : IntegrityError,
  -294 : IntegrityError,
  -295 : IntegrityError,
  -296 : IntegrityError,
  -297 : IntegrityError,
  -313 : ProgrammingError,
  -314 : DataError,
  -401 : ProgrammingError,
  -412 : ProgrammingError,
  -406 : DataError,
  -407 : IntegrityError,
  -413 : DataError,
  -501 : InternalError,
  -502 : InternalError,
  -504 : InternalError,
  -508 : InternalError,
  -510 : InternalError,
  -518 : NotSupportedError,
  -519 : InternalError,
  -530 : IntegrityError,
  -532 : OperationalError,
  -551 : InterfaceError,
  -552 : InterfaceError,
  -553 : InterfaceError,
  -595 : InternalError,
  -596 : InternalError,
  -597 : InterfaceError,
  -598 : InterfaceError,
  -599 : NotSupportedError,
  -600 : NotSupportedError,
  -601 : ProgrammingError,
  -604 : ProgrammingError,
  -605 : ProgrammingError,
  -607 : InternalError,
  -615 : InternalError,
  -616 : IntegrityError,
  -617 : IntegrityError,
  -618 : IntegrityError,
  -625 : DataError,
  -637 : NotSupportedError,
  -660 : IntegrityError,
  -663 : InternalError,
  -664 : InternalError,
  -677 : InternalError,
  -685 : ProgrammingError,
  -689 : InterfaceError,
  -690 : NotSupportedError,
  -691 : DataError,
  -692 : InterfaceError,
  -693 : InterfaceError,
  -694 : InternalError,
  -802 : DataError,
  -803 : IntegrityError,
  -804 : ProgrammingError,
  -806 : NotSupportedError,
  -807 : ProgrammingError,
  -808 : NotSupportedError,
  -809 : NotSupportedError,
  -810 : NotSupportedError,
  -811 : ProgrammingError,
  -816 : InterfaceError,
  -817 : NotSupportedError,
  -820 : NotSupportedError,
  -823 : InternalError,
  -824 : InternalError,
  -825 : InternalError, 
  -826 : InternalError,
  -827 : InternalError,
  -828 : InternalError,
  -829 : InternalError,
  -830 : InternalError,
  -831 : IntegrityError,
  -832 : IntegrityError,
  -833 : NotSupportedError,
  -834 : InternalError,
  -835 : InternalError,
  -836 : InternalError,
  -837 : OperationalError,
  -838 : OperationalError,
  -839 : OperationalError,
  -840 : OperationalError,
  -841 : OperationalError,
  -842 : ProgrammingError,
  -901 : DatabaseError,
  -902 : DatabaseError,
  -904 : OperationalError,
  -906 : InterfaceError,
  -909 : InternalError,
  -911 : InternalError, 
  -913 : InternalError,
  -922 : InterfaceError,
  -923 : InterfaceError,
  -924 : InternalError,
  -926 : InternalError,
  -999 : Error
}


def initialize():
    gvibBase.IBInitExceptions((Warning,
                               Error,
			       InterfaceError,
			       DatabaseError,
			       DataError,
			       OperationalError,
			       IntegrityError,
			       InternalError,
			       ProgrammingError,
			       NotSupportedError,
                               _ExceptionTable))

initialize()
